
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.033.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "read operating system details from standard /etc/os-release file",
  "dist_author" => [
    "Ian Kluft <https://github.com/ikluft>"
  ],
  "dist_name" => "Sys-OsRelease",
  "dist_version" => "0.4.10",
  "license" => "artistic_2",
  "module_name" => "Sys::OsRelease",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "feature" => 0,
    "if" => 0,
    "perl" => "5.022",
    "strict" => 0,
    "utf8" => 0,
    "version" => 0,
    "warnings" => 0
  },
  "test_requires" => {
    "Cwd" => 0,
    "File::Basename" => 0,
    "File::Find" => 0,
    "Pod::Simple::SimpleTree" => 0,
    "Test::More" => 0,
    "YAML" => 0,
    "autodie" => 0
  }
);


my %fallback_build_requires = (
  "Cwd" => 0,
  "File::Basename" => 0,
  "File::Find" => 0,
  "Module::Build" => "0.28",
  "Pod::Simple::SimpleTree" => 0,
  "Test::More" => 0,
  "YAML" => 0,
  "autodie" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
